/*-
 * Copyright (c) 2007
 * Tatsuya Kudoh(CDR/TK),ROYALPANDA.    All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import flash.text.*;
	import flash.utils.*;

	public class Floor extends Bitmap {
		public var type:int;
		public var game:Game;
		public var length:int;

		public var speed:Number = 0;
		public var force:Number = 0;
		public var phase:Number = 0;
		public var floor:int = 0;
		public var blinkDuration:int = 0;
		public var blinkTransit:int = 5;
		public var blinkCount:int = 0;
		public var blinkState:Boolean = true;

		static public const TYPE_NORMAL:int = 0;
		static public const TYPE_FROZEN:int = 1;
		static public const TYPE_FORCE:int = 2;

		public function Floor( aGame:Game, aType:int, aLength:int ):void {
			var b:BitmapData;
			var bm:BitmapData;
			var len:int;
			var r:Rectangle;
			var p:Point;

			type = aType;
			game = aGame;
			length = aLength;

			len = length;
			if( type == TYPE_FORCE )
				len += 32;
			b = Assets.bitmap(Assets.BM_FLOOR);

			bm = new BitmapData(len, 16);
			r = new Rectangle(0, type*16, 32, 16);
			p = new Point(0,0);
			for( ; p.x < len ; p.x += 32 )
				bm.copyPixels(b, r, p);

			super(bm);

			if( type == TYPE_FORCE )
				scrollRect = new Rectangle(0,0,length,16);

		}

		public function move():void {
			if( speed ){
				x += speed;
				if( x < 16 ){
					x = 16;
					speed = -speed;
				}else if( x + length  > game.left - 16 ){
					x = game.left - 16 - length;
					speed = -speed;
				}
			}

			if( force ){
				phase -= force;
				if( phase >= 32 )
					phase -= 32;
				else if( phase < 0 )
					phase += 32;
				scrollRect = new Rectangle(phase, 0, length, 16);
			}

			if( blinkDuration > 0 ){
				blinkCount++;
				if( blinkCount >= blinkDuration ){
					blinkCount = 0;
					blinkState = ! blinkState;

				}else if( blinkCount <= blinkTransit ){
					if( blinkState )
						alpha = blinkCount / blinkTransit;
					else
						alpha = 1 - (blinkCount / blinkTransit);
				}
			}
		}

		public function setBlinkDuration( d:int ):void {
			blinkDuration = d;
			blinkCount = Rnd.next(d - blinkTransit) + blinkTransit;
			blinkState = Rnd.coinToss();
			if( blinkState )
				alpha = 1;
			else
				alpha = 0;
		}
	}
}
