/*-
 * Copyright (c) 2007
 * Tatsuya Kudoh(CDR/TK),ROYALPANDA.    All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import flash.text.*;
	import flash.utils.*;

	public class Player extends Bitmap {
		protected var vx:Number;
		protected var vy:Number;
		protected var dir:int;
		protected var animation:int;
		protected var animationCnt:int;
		protected var game:Game;

		protected var stand:Boolean;

		public var floor:int;

		static protected const DIR_LEFT:int = 0;
		static protected const DIR_RIGHT:int = 1;

		public function Player( aGame:Game ):void {
			super(new BitmapData(32,48));
			vx = 0;
			vy = 0;
			dir = DIR_LEFT;
			game = aGame;
			animationCnt = 0;
			animation = 0;
			changeGraphic();
			floor = 0;
			stand = true;
		}

		protected function getFloor( prevY:Number ):Floor {
			var i:int;
			var fl:Floor;
			var d:Number;

			if( vy < 0 )
				return null;

			for( i = 0 ; i < parent.numChildren ; i++ ){
				fl = parent.getChildAt(i) as Floor;
				if( fl ){
					if( y + height >= fl.y && prevY + height <= fl.y ){
						d = x - fl.x;
						if( d <= fl.width - 4 && d > -28 )
							return fl;
					}
				}
			}
			return null;
		}

		public function move( joy:uint ):void {
			var j:uint;
			var jm:uint;
			var fl:Floor;
			var v:Number;
			var vv:Number;
			var prevY:Number;
			var an:int;
			var d:int;
			var fric:int;

			an = animation;
			d = dir;

			x += vx;
			if( x < 16 ){
				x = 16;
				vx = -vx / 2;
			}else if( x > game.left - 48 ){
				x = game.left - 48;
				vx = -vx / 2;
			}
			prevY = y;
			v = vy;
			if( v < -16 )
				v = -16;
			else if( v > 16 )
				v = 16;
			y += v;

			j = joy & (Joy.LEFT | Joy.RIGHT);
			jm = joy & (Joy.UP | Joy.A);
			fl = getFloor(prevY);
			if( fl != null ){
				stand = true;
				if( fl.floor > floor )
					floor = fl.floor;
				y = fl.y - height;
				v = vx - fl.speed - fl.force;
				vv = v;
				if( fl.type == Floor.TYPE_FROZEN )
					fric = 16;
				else
					fric = 8;
				if( j == Joy.LEFT ){
					dir = DIR_LEFT;
					v -= (16 + vv) / fric;
				}else if( j == Joy.RIGHT ){
					dir = DIR_RIGHT;
					v += (16 - vv) / fric;
				}else if( fl.type != Floor.TYPE_FROZEN ){
					if( v > 1 )
						v -= 1;
					else if( v < -1 )
						v += 1;
					else
						v = 0;
				}
				vx = v + fl.speed + fl.force;
				if( jm ){
					vy = Math.floor(-Math.abs(v) * 1.25 - 20);
					animation = 2;
					animationCnt = 0;
					Snd.play(Assets.SND_JUMP);
					stand = false;
				}else{
					vy = 0;
					if( animation > 1 ){
						animation = 0;
					}else{
						animationCnt++;
						if( animationCnt >= 8 ){
							animationCnt = 0;
							animation ^= 1;
						}
					}
				}
			}else{
				if( stand ){
					Snd.play(Assets.SND_FALL);
					stand = false;
				}
				vv = vx;
				if( j == Joy.LEFT ){
					dir = DIR_LEFT;
					if( vv > 0 )
						vv = 0;
					vv = (16 + vv) / 16;
					if( vv > 0 )
						vx -= vv;
				}else if( j == Joy.RIGHT ){
					dir = DIR_RIGHT;
					if( vv < 0 )
						vv = 0;
					vv = (16 - vv) / 16;
					if( vv > 0 )
						vx += vv;
				}
				vy += 2;
				if( vy < -16 ){
					if( ! jm )
						vy = -16;
				}else{
					if( vy > 0 )
						animation = 3;
				}
			}
			if( an != animation || d != dir )
				changeGraphic();
		}

		protected function changeGraphic():void {
			var b:BitmapData;
			var bm:BitmapData;

			b = Assets.bitmap(Assets.BM_PLAYER);
			bm = bitmapData;
			bm.copyPixels(b, new Rectangle((animation + dir  * 4) * 32, 0, 32, 48), new Point(0,0));
			bitmapData = bm;
				/*  bitmapData.copyPixels() Ɠ삪APlayer ̃oOH */
		}
	}
}
